{% load trionyx %}
<div {% if div.css_id %}id="{{ div.css_id }}"{% endif %}
    class="{{ div.css_class }}" {{ div.flat_attrs|safe }} style="display: none">
        {{ fields|safe }}
</div>
<script>
    runOnReady(function () {
        trionyxFormDepend('#{{ div.css_id }}', {{ div.dependencies|jsonify }});
    });
</script>