{% load trionyx static %}

{% for css in css_files %}
    <link rel="stylesheet" type="text/css" href="{% static css %}">
{% endfor %}

{% for js in js_files %}
    <script src="{% static js %}"></script>
{% endfor %}

<script>
    if ('trionyx_layouts' in window) {
        window.trionyx_layouts['{{ layout.id }}'] = '{{ layout.update_url }}';
    }
</script>

<div id="{{ layout.id }}">
    {% for component in layout.components %}
        {% render_component component %}
    {% endfor %}
</div>