{
    "SECRET_KEY": "{{ secret_key }}",
    "ALLOWED_HOSTS": ["www.{{ app_domain }}", "{{ app_domain }}"],
    "CELERY_BROKER_URL": "amqp://{{ app_user }}:{{ broker_password }}@localhost:5672//",
    "DATABASES": {
        "default": {
            "ENGINE": "django.db.backends.postgresql",
            "NAME": "{{ db_name }}",
		    "USER": "{{ db_username }}",
		    "PASSWORD": "{{ db_password }}",
		    "HOST": "localhost",
		    "PORT": "{{ db_pool_port }}"
        }
    },

    "LOG_DIR": "/home/{{ app_user }}/logs/",

    "EMAIL_HOST": "<SMTP server>",
    "EMAIL_PORT": 25,
    "EMAIL_HOST_USER": "<SMTP user>",
    "EMAIL_HOST_PASSWORD": "{{ smtp_password }}",
    "EMAIL_USE_TLS": false
}