<aside id="trionyx-control-sidebar" class="control-sidebar control-sidebar-light">
    <div id="trionyx-sidebar-top">
        <div id="trionyx-sidebar-controls" style="min-height: 33px;">
            <button class="btn icon-btn btn-sm btn-default pull-right" type="button" onclick="closeSidebar()">
                <span class="fa fa-times"></span>
            </button>

            <div class="btn-group btn-group-sm pull-right" style="margin-right: 5px;">
                <button type="button" class="btn btn-default dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                    <span class="ion ion-ios-more"></span>
                </button>
                <ul class="dropdown-menu"></ul>
            </div>

            <span id="trionyx-sidebar-title"></span>
        </div>
        <div id="trionyx-sidebar-top-content"></div>
    </div>
    <div id="trionyx-sidebar-content"></div>
</aside>
<div class="control-sidebar-bg"></div>