{% for button in header_buttons %}
    <a
       href="{% if button.dialog %}#{% else %}{{ button.url }}{% endif %}"
       {% if button.dialog %}onClick="openDialog('{{ button.url }}', {{ button.dialog_options }})"{% endif %}
       class="btn btn-flat {{ button.type }}"
       target="{{ button.target }}"
    >
        {{ button.label }}
    </a>
{% endfor %}